namespace Microsoft.Services.TestTools.UITesting.Html {
    using System;
    using System.Globalization;

    /// <summary>
    /// Helper class used to generate form input strings
    /// </summary>
    public class StringCreator {
        
        /// <summary>
        /// Randomizer used by the create mathods
        /// </summary>
        private static readonly Random Randomizer = new Random();
        
        /// <summary>
        /// Upper case letters for randomization
        /// </summary>
        private const string Upper = "ABCDEFGHIJKLMNOPQRSTUVXYZ";
        
        /// <summary>
        /// International letters for randomization
        /// </summary>
        private const string International = "";
        
        /// <summary>
        /// Digits for randomization
        /// </summary>
        private const string Numbers  = "0123456789";
        
        /// <summary>
        /// Special characters for randomization
        /// </summary>
        private const string Special = "!#%&/()=@$^~*-.,?[]{}";

        /// <summary>
        /// Lower chase letters used for randomizaton
        /// </summary>
        private const string Lower = "abcdefghijklmnopqrstuvxyz";

        /// <summary>
        /// Create a random string using the source characters provided.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="length">The length.</param>
        /// <returns>The randomized string</returns>
        public string Create(string source, int length) {
            string generated = string.Empty;
            for (int i = 0; i < length; i++) {
                generated += source[Randomizer.Next(source.Length)];
            }

            return generated;
        }

        /// <summary>
        /// Creates a name. 8 chars.
        /// </summary>
        /// <returns>A random name, 8 chars.</returns>
        /// <remarks>Uppercase, Lowercase, and International chars</remarks>
        public string CreateName() {
            return CreateName(8);
        }

        /// <summary>
        /// Creates a name.
        /// </summary>
        /// <param name="length">The length.</param>
        /// <returns>A random name</returns>
        /// <remarks>Uppercase, Lowercase, and International chars</remarks>
        /// <exception cref="ArgumentOutOfRangeException">Length must be greater than zero</exception>
        public string CreateName(int length) {
            if (length < 1) {
                throw new ArgumentOutOfRangeException("length", "must be greater than 0");
            }

            return Create(Upper, 1) + Create(Lower + International, length - 1);
        }

        /// <summary>
        /// Creates a password. 12 chars.
        /// </summary>
        /// <returns>A random, 12 chars, password</returns>
        /// <remarks>Uppercase, Lowercase, Numbers, International and Special chars</remarks>
        public string CreatePassword() {
            return CreatePassword(12);
        }

        /// <summary>
        /// Creates a password.
        /// </summary>
        /// <param name="length">The length.</param>
        /// <returns>A random password</returns>
        /// <remarks>Uppercase, Lowercase, Numbers International and Special chars</remarks>
        /// <exception cref="ArgumentOutOfRangeException">Length must be greater than three</exception>
        public string CreatePassword(int length) {
            if (length < 4) {
                throw new ArgumentOutOfRangeException("length", "must be greater than 3");
            }

            var all = Lower + Upper + International + Numbers + Special;
            
            return 
                Create(Lower, 1) + 
                Create(Upper, 1) + 
                Create(all, length - 4) + 
                Create(Numbers, 1) +
                Create(Special, 1);
        }

        /// <summary>
        /// Creates an email address. 16 chars
        /// </summary>
        /// <returns>A random email address, 16 chars</returns>
        public string CreateEmail() {
            return CreateEmail(16);
        }

        /// <summary>
        /// Creates an email address.
        /// </summary>
        /// <param name="length">The length.</param>
        /// <returns>A random email address</returns>
        /// <exception cref="ArgumentOutOfRangeException">Length must be greater than five</exception>
        public string CreateEmail(int length) {
            if (length < 6) {
                throw new ArgumentOutOfRangeException("length", "must be greater than 5");
            }

            int parts = (length - 4) / 2;
            
            return string.Format(
                "{0}@{1}.{2}",
                Create(Lower, parts), 
                Create(Lower, parts), 
                Create(Lower, 2));
        }

        /// <summary>
        /// Creates an amount. Two decimals.
        /// </summary>
        /// <returns>An amount. Less than 100</returns>
        public string CreateAmount() {
            return CreateAmount(100, 2);
        }

        /// <summary>
        /// Creates an amount.
        /// </summary>
        /// <param name="multiplyer">The multiplyer.</param>
        /// <param name="decimals">The decimals.</param>
        /// <returns>An amount</returns>
        public string CreateAmount(int multiplyer, int decimals) {
            return CreateAmount(multiplyer, decimals, CultureInfo.CurrentCulture);
        }

        /// <summary>
        /// Creates an amount.
        /// </summary>
        /// <param name="multiplyer">The multiplyer.</param>
        /// <param name="decimals">The decimals.</param>
        /// <param name="formatProvider">The format provider.</param>
        /// <returns>An amount</returns>
        public string CreateAmount(int multiplyer, int decimals, IFormatProvider formatProvider) {
            return Math.Round(Randomizer.NextDouble() * multiplyer, decimals).ToString(formatProvider);
        }

        /// <summary>
        /// Creates a number. 4 digits
        /// </summary>
        /// <returns>A random 4 digit number</returns>
        public string CreateNumeric() {
            return CreateNumeric(4);
        }

        /// <summary>
        /// Creates a number.
        /// </summary>
        /// <param name="length">The length.</param>
        /// <returns>A random number</returns>
        public string CreateNumeric(int length) {
            return Create(Numbers, length);
        }
    }
}